@@ -22,6 +22,7 @@ class SimplificationManager:
             "LShR": self.lshr_simplifier,
             "__eq__": self.eq_simplifier,
             "__ne__": self.ne_simplifier,
+            "__ge__": self.ge_simplifier,
             "__or__": self.bitwise_or_simplifier,
             "__and__": self.bitwise_and_simplifier,
             "__xor__": self.bitwise_xor_simplifier,
@@ -33,6 +34,7 @@ class SimplificationManager:
             "fpToIEEEBV": self.fptobv_simplifier,
             "fpToFP": self.fptofp_simplifier,
             "StrReverse": self.str_reverse_simplifier,
+            "__invert__": self.invert_simplifier,
         }
 
     def simplify(self, op, args):
@@ -198,6 +200,17 @@ class SimplificationManager:
         if a.op == "BVV" and b.op != "BVV":
             return b == a
 
+        # expr - c1 == c2    ->   expr == c1 + c2
+        if a.op == "__sub__" and len(a.args) == 2 and a.args[1].op == "BVV" and b.op == "BVV":
+            return a.args[0] == a.args[1] + b
+
+        # 1 ^ expr == 0     ->   expr == 1
+        if a.op == "__xor__" and b.op == "BVV" and b.args[0] == 0 and len(a.args) == 2:
+            if a.args[1].op == "BVV" and a.args[1].args[0] == 1:  # expr ^ 1 == 0
+                return a.args[0] == 1
+            elif a.args[0].op == "BVV" and a.args[0].args[0] == 1:  # 1 ^ expr == 0
+                return a.args[1] == 1
+
         # TODO: all these ==/!= might really slow things down...
         if a.op == "If":
             if a.args[1] is b and ast.all_operations.is_true(a.args[2] != b):
@@ -283,6 +296,13 @@ class SimplificationManager:
             # elif b._claripy.is_true(b.args[1] == a) and b._claripy.is_true(b.args[2] == a):
             # 	  return b._claripy.false
 
+        # 1 ^ expr != 0     ->   expr == 0
+        if a.op == "__xor__" and b.op == "BVV" and b.args[0] == 0 and len(a.args) == 2:
+            if a.args[1].op == "BVV" and a.args[1].args[0] == 1:
+                return a.args[0] != 1
+            elif a.args[0].op == "BVV" and a.args[0].args[0] == 1:
+                return a.args[1] != 1
+
         # Masking and comparing against a constant
         simp = SimplificationManager.and_mask_comparing_against_constant_simplifier(operator.__ne__, a, b)
         if simp is not None:
@@ -311,6 +331,13 @@ class SimplificationManager:
                 if ast.all_operations.is_true(a_bit != b_bit):
                     return ast.all_operations.true
 
+    @staticmethod
+    def ge_simplifier(a, b):
+        # ZeroExt/Concat and comparing against a constant
+        simp = SimplificationManager.zeroext_comparing_against_simplifier(operator.__ge__, a, b)
+        if simp is not None:
+            return simp
+
     @staticmethod
     def bv_reverse_simplifier(body):
         if body.op == "Reverse":
@@ -374,6 +401,15 @@ class SimplificationManager:
         if len(new_args) < len(args):
             return ast.all_operations.And(*new_args)
 
+        # a >= c && a != c    ->   a>c
+        if len(args) == 2:
+            a = args[0]
+            b = args[1]
+            if len(a.args) >= 2 and len(b.args) >= 2:
+                if a.args[0] is b.args[0] and a.args[1] is b.args[1]:
+                    if a.op == "__ge__" and b.op == "__ne__":
+                        return ast.all_operations.UGT(a.args[0], a.args[1])
+
         flattened = SimplificationManager._flatten_simplifier("And", SimplificationManager._deduplicate_filter, *args)
         if flattened is None:
             return None
@@ -632,7 +668,12 @@ class SimplificationManager:
             return tuple(res)
 
         return SimplificationManager._flatten_simplifier(
-            "__xor__", _flattening_filter, a, b, *args, initial_value=ast.all_operations.BVV(0, a.size())
+            "__xor__",
+            _flattening_filter,
+            a,
+            b,
+            *args,
+            initial_value=ast.all_operations.BVV(0, a.size()),
         )
 
     @staticmethod
@@ -687,6 +728,22 @@ class SimplificationManager:
                     # yes!
                     return ast.all_operations.ZeroExt(a.args[0].size(), a.args[1])
 
+            # if(cond0, 1, 0) & if(cond1, 1, 0)  ->  if(cond0 & cond1, 1, 0)
+            if a.op == "If" and b.op == "If":
+                if (
+                    (a.args[1] == ast.all_operations.BVV(1, 1)).is_true()
+                    and (a.args[2] == ast.all_operations.BVV(0, 1)).is_true()
+                    and (b.args[1] == ast.all_operations.BVV(1, 1)).is_true()
+                    and (b.args[2] == ast.all_operations.BVV(0, 1)).is_true()
+                ):
+                    cond0 = a.args[0]
+                    cond1 = b.args[0]
+                    return ast.all_operations.If(
+                        cond0 & cond1,
+                        ast.all_operations.BVV(1, 1),
+                        ast.all_operations.BVV(0, 1),
+                    )
+
         return SimplificationManager._flatten_simplifier(
             "__and__", SimplificationManager._deduplicate_filter, a, b, *args
         )
@@ -845,6 +902,17 @@ class SimplificationManager:
             else:
                 return reduce(getattr(operator, val.op), all_args)
 
+        #  (if cond then 1 else 0)[0:0]  ->  if(cond then 1[0:0] else 0[0:0])
+        if val.op == "If":
+            ifcond, iftrue, iffalse = val.args
+            if iftrue.op == "BVV" and iffalse.op == "BVV":
+                # extract from iftrue and iffalse
+                return ast.bool.If(ifcond, iftrue[high:low], iffalse[high:low])
+
+        # invert(expr)[0:0]   ->   invert(expr[0:0])
+        if val.op == "__invert__":
+            return ast.BV.__invert__(val.args[0][high:low])
+
     # oh gods
     @staticmethod
     def fptobv_simplifier(the_fp):
@@ -920,6 +988,12 @@ class SimplificationManager:
     def str_reverse_simplifier(arg):
         return arg
 
+    @staticmethod
+    def invert_simplifier(expr):
+        # ~ if(cond then 1 else 0)  ->  if(cond, ~1, ~0)  ->    if(!cond, 1,0)
+        if expr.op == "If" and expr.args[1].op == "BVV" and expr.args[1].args[0] == 1 and expr.args[2].args[0] == 0:
+            return ast.bool.If(ast.all_operations.Not(expr.args[0]), expr.args[1], expr.args[2])
+
     @staticmethod
     def and_mask_comparing_against_constant_simplifier(op, a, b):
         """
@@ -1029,28 +1103,36 @@ class SimplificationManager:
         """
         This simplifier handles the following cases:
 
-            ZeroExt(n, A) == b, and
-            ZeroExt(n, A) != b
+            ZeroExt(n, A) == b,
+            ZeroExt(n, A) != b, and
+            ZeroExt(n, A) >= b
 
-        If the high bits of b are all zeros (in case of __eq__) or have at least one ones (in case of __ne__), ZeroExt
-        can be eliminated.
+        If the high bits of b are all zeros (in case of ==, !=, and >=) or have at least one ones (in case of !=),
+        ZeroExt can be eliminated.
         """
-        if op in {operator.__eq__, operator.__ne__} and b.op == "BVV":
-            # check if the high bits of b are zeros
+        if op in {operator.__eq__, operator.__ne__, operator.__ge__} and b.op == "BVV":
             if a.op == "ZeroExt":
+                # check if the high bits of b are zeros
                 a_zeroext_bits = a.args[0]
-            elif a.op == "Concat" and len(a.args) == 2 and (a.args[0] == 0).is_true():
-                a_zeroext_bits = a.args[0].size()
-            else:
-                return None
+                b_highbits = b[b.size() - 1 : b.size() - a_zeroext_bits]
+                if (b_highbits == 0).is_true():
+                    # we can get rid of ZeroExt
+                    return op(a.args[1], b[b.size() - a_zeroext_bits - 1 : 0])
+                elif (b_highbits == 0).is_false():
+                    # unsat
+                    return ast.all_operations.false if op is operator.__eq__ else ast.all_operations.true
 
-            b_highbits = b[b.size() - 1 : b.size() - a_zeroext_bits]
-            if (b_highbits == 0).is_true():
-                # we can get rid of ZeroExt
-                return op(a.args[1], b[b.size() - a_zeroext_bits - 1 : 0])
-            elif (b_highbits == 0).is_false():
-                # unsat
-                return ast.all_operations.false if op is operator.__eq__ else ast.all_operations.true
+            if (
+                a.op == "Concat" and len(a.args) == 2 and a.args[0].op == "BVV" and a.args[0].args[0] == 0
+            ):  # make sure high bits of a are zeros
+                a_zero_bits = a.args[0].args[1]
+                b_highbits = b[b.size() - 1 : b.size() - a_zero_bits]
+                if (b_highbits == 0).is_true():
+                    # we can get rid of Concat
+                    return op(a.args[1], b[b.size() - a_zero_bits - 1 : 0])
+                elif (b_highbits == 0).is_false():
+                    # unsat
+                    return ast.all_operations.false if op is operator.__eq__ else ast.all_operations.true
 
         return None
 
